/// <reference path="interfaces.d.ts" />
/// <reference path="permission.d.ts" />
import { IRoleDef, IRoleDefs, IScopeDef, IResourceDef, ICondition, IConstraint, IContext, IEffect, IFieldGenerator, IPermission } from './interfaces';
export declare function All(): boolean;
export declare class RBACPlus {
    roles: IRoleDefs;
    constructor(roles?: IRoleDefs);
    grant(roleName: string): Role;
    deny(roleName: string): Role;
    can(roleNames: string | string[], scope: string, context?: IContext): Promise<IPermission>;
    private getWithDefaultWildcard<T>(key, obj);
    private canRole(roleName, resourceName, actionName, field, context, permission);
    private ensureRole(roleName);
    private testCondition(condition, context);
    private canAction(action, actionPath, field, context, permission);
    private canScope(scope, scopePath, field, context, permission);
    private canGrantScope(scope, scopePath, field, context, permission);
    private canDenyScope(scope, scopePath, field, context, permission);
    private testScope(scope, scopePath, field, context);
    private permissionPath(scopePath, field, fieldTest, condition);
    private generateFields(scope, context);
}
export declare class Role extends RBACPlus {
    readonly roleName: string;
    readonly effect: IEffect;
    constructor(roles: IRoleDefs, roleName: string, effect: IEffect);
    inherits(roleName: string): this;
    resource(resourceName: string): Resource;
    scope(scope: string): Scope;
    protected readonly _role: IRoleDef;
}
export declare class Resource extends Role {
    readonly resourceName: string;
    constructor(roles: IRoleDefs, roleName: string, effect: IEffect, resourceName: string);
    action(actionName: string): Scope;
    readonly create: Scope;
    readonly read: Scope;
    readonly update: Scope;
    readonly delete: Scope;
    protected readonly _resource: IResourceDef;
}
export declare class Scope extends Resource {
    readonly actionName: string;
    readonly scopeIndex: number;
    constructor(roles: IRoleDefs, roleName: string, effect: IEffect, resourceName: string, actionName: string, scopeIndex: number);
    withConstraint(constraint: IConstraint): Scope;
    onDynamicFields(fieldGen: IFieldGenerator): Scope;
    onFields(...fieldNames: string[]): Scope;
    where(...conditions: ICondition[]): Scope;
    or(...conditions: ICondition[]): Scope;
    and(...conditions: ICondition[]): Scope;
    readonly condition: ICondition;
    private _condition;
    protected readonly _scope: IScopeDef;
}
